#
#  Humerus - Choose Level Screen
#

import os
from albow.widget import Widget
from albow.controls import Label, Column
from albow.palette_view import PaletteView
from albow.resource import get_font
from screen import Screen
from game import filename_to_level_name

title_font = get_font(18, "VeraBd.ttf")
title_color = (255, 255, 255)
level_list_width = 400
level_list_rows = 20
level_list_font = get_font(15, "VeraBd.ttf")
tutorial_level_color = (0, 128, 255)
completed_level_color = (128, 255, 0)
next_level_color = (255, 255, 0)

class ChooseLevelScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		title = Label("Available Levels", font = title_font, fg_color = title_color)
		self.level_list = LevelListView(shell)
		box = Column([title, self.level_list], spacing = 10)
		box.rect.center = self.rect.center
		self.add(box)
	
	def enter_screen(self):
		self.level_list.refresh()


class LevelListView(PaletteView):
	#  items   [string]   Choosable level names, last is "" if all completed
	
	def __init__(self, shell):
		w, h = level_list_font.size("X")
		PaletteView.__init__(self, cell_size = (level_list_width, h),
			nrows = level_list_rows, ncols = 1, scrolling = True)
		self.shell = shell
		self.game = shell.game
	
	def refresh(self):
		tutorials = self.game.get_current_playing_level_list(
			tutorial = True, regular = False)
		names = [os.path.basename(p) for p in tutorials]
		names.extend(self.game.completed_levels[:])
		names.sort()
		next_path = self.game.next_uncompleted_level()
		if next_path:
			next_name = os.path.basename(next_path)
			names.append(next_name)
		else:
			names.append("")
		self.items = names
	
	def num_items(self):
		items = self.items
		n = len(items)
		if items and items[-1] == "":
			n -= 1
		return n

	def draw_item(self, surf, i, frame):
		items = self.items
		filename = items[i]
		text = filename_to_level_name(filename)
		if text:
			if i == len(items) - 1:
				color = next_level_color
			elif filename.startswith("+"):
				color = tutorial_level_color
			else:
				color = completed_level_color
			buf = level_list_font.render(text, True, color)
			surf.blit(buf, frame)

	def click_item(self, i, e):
		filename = self.items[i]
		level_dir = self.game.get_current_playing_level_dir()
		path = os.path.join(level_dir, filename)
		self.shell.start_level_file(path)
	
	def item_is_selected(self, i):
		return False
