#
#   Humerus - File Reader
#

class Reader(object):

	def __init__(self, f, path = "Unknown file"):
		self.f = f
		self.path = path
		self.lineno = 0
		self.eof = False
		self.next_line()
	
	def next_line(self):
		line = self.f.readline()
		self.line = line
		token = ""
		args = None
		if line:
			self.lineno += 1
			parts = line.split(" ", 1)
			if parts:
				token = parts.pop(0)
			if parts:
				try:
					args = eval(parts[0])
				except Exception, e:
					self.error(str(e))
		else:
			self.eof = True
		self.token = token.strip()
		self.args = args

	def error(self, mess):
		raise EnvironmentError("%s, line %s: Invalid file format: %s"
			% (self.path, self.lineno, mess))
	
	def check(self, token):
		if self.token <> token:
			self.error("Expected '%s', got '%s'" % (token, self.token))

	def expect(self, token):
		self.check(token)
		self.next_line()

	def looking_at(self, token):
		if self.eof:
			self.error("Unexpected end of file")
		return self.token == token
	
	def pop_arg(self, *dflt):
		args = self.args
		if not args:
			if dflt:
				return dflt[0]
			self.error("Not enough arguments")
		return args.pop(0)

	def get_arg(self, conv, *dflt):
		return self.convert(self.pop_arg(*dflt), conv)

	def get_tuple(self, *convs):
		args = self.get_arg(tuple)
		return self.convert_sequence(args, convs)

	def get_tuple_list(self, *convs):
		items = self.pop_arg()
		if not isinstance(items, list):
			self.error("Expected a list")
		for t in items:
			if not isinstance(t, tuple):
				self.error("Expected a tuple")
			yield self.convert_sequence(t, convs)
	
	def convert_sequence(self, args, convs):
		if len(args) <> len(convs):
			self.error("Wrong number of items in sequence")
		return [self.convert(arg, conv) for conv, arg in zip(convs, args)]

	def convert(self, arg, conv):
		try:
			return conv(arg)
		except (TypeError, ValueError), e:
			self.error(str(e))
